package rca;

import Vector :: *;

// Full adder
function Tuple2 #(Bit#(1), Bit #(1)) fa (Bit#(1) a, Bit#(1) b, Bit#(1) cin);
  Bit#(1) s = a ^ b ^ cin;
  Bit#(1) cout = (a & b) | (b & cin) | (a & cin);
  return tuple2 (cout, s);
endfunction

typedef Tuple2 #(Bit #(1), Bit #(1))  B2;

// Ripple carry adder

// See BSV Reference Guide "Vectors" section for: sscanl, zipWith, map
// See BSV Reference Guide "Tuples" section for: tuple2, tpl_1, tpl_2
// See BSV Reference Guide "Bits typeclass" section for: unpack, pack

function Bit#(TAdd#(w, 1)) rca (Bit#(w) a, Bit#(w) b, Bit#(1) cin);
   Vector #(w, Bit #(1)) v_a = unpack (a);
   Vector #(w, Bit #(1)) v_b = unpack (b);

   function B2 scanfn (B2 cin_and_v, B2 ab) = fa (tpl_1 (ab), tpl_2 (ab), tpl_1 (cin_and_v));

   Vector #(w, B2) vpairs = sscanl (scanfn,
				    tuple2 (cin, 0),
				    zipWith (tuple2, v_a, v_b));

   Vector #(w, Bit #(1)) x = map (tpl_2, vpairs);
   Bit #(1) cout = tpl_1 (vpairs [valueOf (w)-1]);

   return { cout, pack (x) };
endfunction

endpackage
